package scanner;

import static util.Characters.*;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Removes Lisp comments from an input stream.
 */
public class LispFilterInputStream extends FilterInputStream {

    private boolean isInQuotedString;
    private int previousCharacter;
    private int nextCharacter;

    public LispFilterInputStream(InputStream underlyingInputStream) {
        super(underlyingInputStream);

        isInQuotedString = false;
        previousCharacter = 0;
        nextCharacter = 0;
    }

    @Override
    public int read() throws IOException {
        readNextCharacter();

        if (haveEnteredComment())
            consumeAllBytesInComment();

        return nextCharacter;
    }

    private void readNextCharacter() throws IOException {
        previousCharacter = nextCharacter;
        nextCharacter = super.read();

        if (haveEncounteredStringBoundary())
            isInQuotedString = !isInQuotedString;
    }

    private boolean haveEncounteredStringBoundary() {
        return (previousCharacter != BACKSLASH) && (nextCharacter == DOUBLE_QUOTE);
    }

    private boolean haveEnteredComment() {
        return (nextCharacter == SEMICOLON) && (!isInQuotedString);
    }

    private void consumeAllBytesInComment() throws IOException {
        while (stillInComment())
            nextCharacter = super.read();
    }

    private boolean stillInComment() {
        return (nextCharacter != NEWLINE) && (nextCharacter != EOF);
    }

}
